function [thr,pp_s,pp_c,bb_s,bb_c,y_av_s,y_av_c,dirac_s,dirac_c,share_s,p0] = inputs_check_indiv(data)

   
%% 1. Average income/wealth
[y_av]= data(1,2); %average income
if ~isscalar(y_av)||~isfloat(y_av)||~(y_av>0)
    error('Average income should be a positive real number.')
end
data = data(3:size(data,1),:);

if data(1,1)~=0
    error('The first threshold should be 0 to enter the share of singles for lower incomes.')
end
    
if size(data,2)<4
    error('Data should include the share of singles for each bracket')
elseif size(data,2)==4
    data=[data ones(size(data,1),1)];
    disp('It will be assumed that, within each bracket, the mean income of the singles is the same as the mean income of the couples.')
end
dirac = NaN;

sh = data(:,4);

if ~min(isnumeric(sh))||~min(sh>=0)||~min(sh<=1)||max(isnan(sh))
    error('The share of singles should be between 0 and 1.')
end
    
rr = data(:,5);
if ~min(isnan(rr))&&(~min(isnumeric(rr))||~min(rr>0))
    error('Ratios of average incomes whithin each bracket should be positive.')
elseif max(isnan(rr))
    rr(isnan(rr)) = ones(sum(isnan(rr)),1);
    disp('For the brackets where ratios of average incomes are not provided by the user, it will be assumed that, within each bracket, the mean income of the singles is the same as the mean income of the couples.')
end


data=[data(:,1:3) NaN(size(data,1),1)];


if data(1,1)<0
    error('All thresholds should be nonnegative.')
else
    if (data(1,1)~=data(2,1))&&~((data(1,1)==0)&&(data(1,2)>0)&&(data(2,1)>0)) % if no dirac
        if (data(1,1)==0)&&(data(1,2)==0)&&(data(2,1)>0) % if implied dirac
            data = data(2:size(data,1),:);
        end
        [thr,pp,bb] = tab_check(data);

        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+pp(1)*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)))
            error('Average income is not consistent with Pareto coefficients.')
        end
    else % if dirac
        if (data(1,1)==0)&&(data(1,2)>0)&&(data(2,1)>0) % if implied dirac
            error('The share of singles should be provided for the dirac.')
        end
        if data(1,1)>=data(3,1)
            error('Thresholds should be in ascending order.')
        elseif data(2,2)>=data(3,2)
            error('Percentiles should be in ascending order.')
        elseif ~(data(1,2)==0||isnan(data(1,2)))
            error('If there is a dirac at the bottom of the distribution, the first percentile should be 0.')
        end
        if isnan(data(1,2))
            data(1,2)=0;
        end
        
        dirac = [data(1,1) data(2,2)-data(1,2)];
        if (dirac(1)==0)||(isnan(data(2,3))&&isnan(data(2,4)))
        data = data(3:size(data,1),:);
        [thr,pp,bb] = tab_check(data);
        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+(pp(1)-dirac(1,2))*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)))
            error('Dirac is not consistent with average income and Pareto coefficients.')
        end
        else
        data = data(2:size(data,1),:);
        [thr,pp,bb] = tab_check(data);
        if round(y_av)~=round((1-pp(1))*thr(1)*bb(1)+dirac(1)*dirac(2))
            error('Dirac is not consistent with average income and Pareto coefficients.')
        end
        end
    end
end
    
if ~(length(dirac)==1)
    p0 = dirac(1,2);
else
    p0 = NaN;
end

%% Singles and couples

	y = (1-pp).*thr.*bb;
	y1 = [y(2:end); 0];
	pp1 = [pp(2:end); 1];
	mu = (y-y1)./(pp1-pp);
    

if ~(length(dirac)==1) %if dirac
    dirac_s = [dirac(1) sh(1)*dirac(2)];
    dirac_c = [dirac(1) (1-sh(1))*dirac(2)];

    pp1 = [pp(2:end); 1];
    
    
    pp_s = [dirac_s(1,2)+sh(2)*(pp(1)-dirac_s(1,2)-dirac_c(1,2)); dirac_s(1,2)+sh(2)*(pp(1)-dirac_s(1,2)-dirac_c(1,2))+cumsum((pp1(1:end-1)-pp(1:end-1)).*sh(3:end-1))];
    share_s = dirac_s(1,2)+sh(2)*(pp(1)-dirac_s(1,2)-dirac_c(1,2))+sum((pp1-pp).*sh(3:end));
    pp_s = pp_s/share_s;
    dirac_s(1,2) = dirac_s(1,2)/share_s;
    
    pp_c = [dirac_c(1,2)+(1-sh(2))*(pp(1)-dirac_s(1,2)-dirac_c(1,2)); dirac_c(1,2)+(1-sh(2))*(pp(1)-dirac_s(1,2)-dirac_c(1,2))+cumsum((pp1(1:end-1)-pp(1:end-1)).*(1-sh(3:end-1)))];
    pp_c = pp_c/(1-share_s);
    dirac_c(1,2) = dirac_c(1,2)/(1-share_s);

    
    mu_s = mu./(sh(3:end)+(1-sh(3:end))./rr(3:end));
    mu_c = mu./(rr(3:end).*sh(3:end)+(1-sh(3:end)));
    
    bb_s = ones(size(mu_s));
    bb_s(end) = mu_s(end)/thr(end);
    
    bb_c = ones(size(mu_c));
    bb_c(end) = mu_c(end)/thr(end);

    for i=(length(bb_s)-1):(-1):1
        bb_s(i) = ((1-pp_s(i+1))*thr(i+1)*bb_s(i+1)+(pp_s(i+1)-pp_s(i))*mu_s(i))/((1-pp_s(i))*thr(i));
    end
    
    for i=(length(bb_c)-1):(-1):1
        bb_c(i) = ((1-pp_c(i+1))*thr(i+1)*bb_c(i+1)+(pp_c(i+1)-pp_c(i))*mu_c(i))/((1-pp_c(i))*thr(i));
    end
    
    mean1 = (y_av-(1-pp(1))*thr(1)*bb(1))/(pp(1)-dirac_s(1,2)-dirac_c(1,2));
    y_av_s = (pp_s(1)-dirac_s(1,2))*(mean1/(sh(2)+(1-sh(2))/rr(2)))+(1-pp_s(1))*thr(1)*bb_s(1);
    y_av_c = (pp_c(1)-dirac_c(1,2))*(mean1/(rr(2)*sh(2)+(1-sh(2))))+(1-pp_c(1))*thr(1)*bb_c(1);

else %if no dirac
    dirac_s = NaN;
    dirac_c = NaN;

    pp1 = [pp(2:end); 1];
    
    pp_s = [sh(1)*pp(1); sh(1)*pp(1)+cumsum((pp1(1:end-1)-pp(1:end-1)).*sh(2:end-1))];
    share_s = sh(1)*pp(1)+sum((pp1(1:end)-pp(1:end)).*sh(2:end));
    pp_s = pp_s/share_s;
    
    pp_c = [(1-sh(1))*pp(1); (1-sh(1))*pp(1)+cumsum((pp1(1:end-1)-pp(1:end-1)).*(1-sh(2:end-1)))];
    pp_c = pp_c/(1-share_s);
    
    mu_s = mu./(sh(2:end)+(1-sh(2:end))./rr(2:end));
    mu_c = mu./(rr(2:end).*sh(2:end)+(1-sh(2:end)));
    
    bb_s = ones(size(mu_s));
    bb_s(end) = mu_s(end)/thr(end);
    
    bb_c = ones(size(mu_c));
    bb_c(end) = mu_c(end)/thr(end);

    for i=(length(bb_s)-1):(-1):1
        bb_s(i) = ((1-pp_s(i+1))*thr(i+1)*bb_s(i+1)+(pp_s(i+1)-pp_s(i))*mu_s(i))/((1-pp_s(i))*thr(i));
    end
    
    for i=(length(bb_c)-1):(-1):1
        bb_c(i) = ((1-pp_c(i+1))*thr(i+1)*bb_c(i+1)+(pp_c(i+1)-pp_c(i))*mu_c(i))/((1-pp_c(i))*thr(i));
    end
    
    mean1 = (y_av-(1-pp(1))*thr(1)*bb(1))/pp(1);
    y_av_s = pp_s(1)*(mean1/(sh(2)+(1-sh(2))/rr(2)))+(1-pp_s(1))*thr(1)*bb_s(1);
    y_av_c = pp_c(1)*(mean1/(rr(2)*sh(2)+(1-sh(2))))+(1-pp_c(1))*thr(1)*bb_c(1);
end



function [thr,pp,bb] = tab_check(tab)
	thr = tab(:,1); % thresholds
	pp  = tab(:,2); % percentiles
	bb  = tab(:,3); % inverted Pareto coefficients
	av  = tab(:,4); % average values within brackets
        
	y = (1-pp).*thr.*bb;
	y1 = [y(2:end); 0];
	pp1 = [pp(2:end); 1];
	mu = (y-y1)./(pp1-pp);
	
    if (thr(1)==0)&&((pp(1)~=0)&&(~isnan(pp(1))))
        error('If 0 is not a dirac and if there is no negative thresholds, please do not include it. If 0 a dirac, write it twice.')
    end
    if ~issorted(thr)||~isnumeric(thr)
        error('Thresholds should be numeric values in ascending order.')
	elseif ~(length(unique(thr))==length(thr))
        error('A dirac can only be at the bottom of the distribution.')
    end
    if ~issorted(pp)||~isnumeric(pp)
        error('Percentiles should be numeric numbers in ascending order.')
	elseif ~min(pp>=0)||~min(pp<1)
        error('Percentiles should be between 0 and 1.')
	elseif ~(length(unique(pp))==length(pp))
        error('A dirac can only be at the bottom of the distribution.')
    end
	if ~min(isnan(bb))
        if ~isnumeric(bb)||~min(bb(~isnan(bb))>1)
            error('Pareto coefficients should be numeric numbers greater than 1.')
        end
    end
	if ~min(mu(~isnan(mu))>thr(~isnan(mu)))||~min(max((thr(2:size(thr,1),1)>mu(1:(size(mu,1)-1),1)),isnan(mu(1:(size(mu,1)-1),1))))
        error('Implied mean values within brackets are not consistent.')
	elseif ~min(isnan(av))
        if ~min(isnumeric(av))||~min(av(~isnan(av))>thr(~isnan(av)))||~min(max((thr(2:size(thr,1),1)>av(1:(size(av,1)-1),1)),isnan(av(1:(size(av,1)-1),1))))
        	error('Given average values are not consistent.')
        end
    end
	if max(~min(~isnan(av),~isnan(mu)))
        if ~min(round(av(min(~isnan(av),~isnan(mu))))==round(mu(min(~isnan(av),~isnan(mu)))))
            error('Average values implied by Pareto coefficients and given average values are not consistent.')
        end
    end
	if max(min(isnan(bb(:)),isnan(av(:))))
        error('A Pareto coefficient or an average value should be provided for each bracket.')
    end
    
    av(isnan(av)) = mu(isnan(av));
    
	if thr(1)==0
        thr = thr(2:end);
         pp = pp(2:end);
         bb = bb(2:end);
         av = av(2:end);
         mu = mu(2:end);
    end
    
    if isnan(bb(end))
        bb(end) = av(end)/thr(end);
    end
    for i=(length(bb)-1):(-1):1
        if isnan(bb(i))
            bb(i) = ((1-pp(i+1))*thr(i+1)*bb(i+1)+(pp(i+1)-pp(i))*av(i))/((1-pp(i))*thr(i));
        end
    end

disp('All inputs are ok.')